package clayout;

import java.awt.*;


public class VerticalLayoutManager implements java.awt.LayoutManager, java.io.Serializable {

    // constant
    public static final int TOP 	= 0;
    public static final int MIDDLE 	= 1;
    public static final int BOTTOM 	= 2;
    public static final int DEFAULT_VERTICAL_GAP = 5;
    public static final int DEFAULT_HORIZONTAL_GAP = 5;
    public static final int DEFAULT_ALIGNMENT = MIDDLE;

    // state
    private int align;
    private int hgap;
    private int vgap;


     /**
      *
      */
     public VerticalLayoutManager() {
      this(DEFAULT_ALIGNMENT,DEFAULT_HORIZONTAL_GAP,DEFAULT_VERTICAL_GAP);
     }


    /**
     *
     * @param align
     */
    public VerticalLayoutManager(int align) {
     this(DEFAULT_ALIGNMENT,DEFAULT_HORIZONTAL_GAP,DEFAULT_VERTICAL_GAP);
    }


    /**
     *
     * @param align
     * @param hgap
     * @param vgap
     */
    public VerticalLayoutManager(int align, int hgap, int vgap) {
     setAlignment(align);
     setHorizontalGap(hgap);
     setVerticalGap(vgap);
    }


    public void setHorizontalGap(int hGap){
     if( hGap < 0) throw new IllegalArgumentException(hGap+" is not a legal value");
     hgap = hGap;
    }


    public void setVerticalGap(int vGap){
     if( vGap < 0) throw new IllegalArgumentException(vGap+" is not a legal value");
     vgap = vGap;
    }


    public void setAlignment(int align){
     if ( ( align < TOP ) || ( align > BOTTOM ) ) throw new IllegalArgumentException(align+" is not a legal value");
     this.align = align;
    }

    /**
     *
     * @param name
     * @param comp
     */
    public void addLayoutComponent(String name, Component comp){}


    /**
     *
     * @param comp
     */
    public void removeLayoutComponent(Component comp){}



    /**
     *
     * @param target
     * @return
     */
    public Dimension preferredLayoutSize(Container target) {
		Dimension tarsiz = new Dimension(0, 0);
                Component [] components = target.getComponents();
		int len = components.length;

                for (int i = 0 ; i < len; i++) {
	    	 Component m = components[i];

                    if (m.isVisible()) {
                                    Dimension d = m.getPreferredSize();
                                    tarsiz.width = Math.max(tarsiz.width, d.width);
                                    if (i > 0) {
                                    tarsiz.height += hgap;
                                    }
                                    tarsiz.height += d.height;
                    }
                  }

		Insets insets = target.getInsets();
		tarsiz.width += insets.left + insets.right + hgap * 2;
		tarsiz.height += insets.top + insets.bottom + vgap * 2;

          return tarsiz;
     }



    /**
     *
     * @param target
     * @return
     */
    public Dimension minimumLayoutSize(Container target) {
		Dimension tarsiz = new Dimension(0, 0);
                Component [] components = target.getComponents();
		int len = components.length;

		for (int i = 0 ; i < len ; i++) {

                  Component m = components[i];

                    if (m.isVisible()) {
                                    Dimension d = m.getMinimumSize();
                                    tarsiz.width = Math.max(tarsiz.width, d.width);
                                    if (i > 0) {
                                    tarsiz.height += vgap;
                                    }
                                    tarsiz.height += d.height;
                    }
                  }

		Insets insets = target.getInsets();
		tarsiz.width += insets.left + insets.right + hgap * 2;
		tarsiz.height += insets.top + insets.bottom + vgap * 2;

      return tarsiz;
    }



    /**
     *
     * @param target
     */
    public void layoutContainer(Container target) {
		Insets insets = target.getInsets();
		int maxheight = target.getSize().height - (insets.top + insets.bottom + (vgap * 2));
		int maxwidth = target.getSize().width - (insets.left + insets.right + (hgap * 2));
		int numcomp = target.getComponentCount();
		int x = insets.left + hgap, y = 0;
		int colw = 0, start = 0;

		for (int i = 0 ; i < numcomp ; i++) {
	    	Component m = target.getComponent(i);
	    	if (m.isVisible()) {

                                Dimension d = m.getPreferredSize();
				m.setSize(d.width, d.height);

				if ( y > maxheight ){
					moveComponents(target, x, insets.top + vgap,
								   colw, maxheight- y, start, i);
					y = d.height;
					x += hgap + colw;
					colw = d.width;
					start = i;
				}else{
					if ( y > 0 ) y+= vgap;
					y += d.height;
					colw = Math.max(colw, d.width);
				}
			}
		}

        moveComponents(target, x, insets.top + vgap, colw, maxheight - y, start, numcomp);
    }



    /**
     *
     * @param target
     * @param x
     * @param y
     * @param width
     * @param height
     * @param first
     * @param last
     */
    private void moveComponents(Container target,
                                      int x,
                                      int y,
	                              int width,
                                      int height,
                                      int first,
                                      int last){

		switch(align){
                 case MIDDLE: y += (height / 2); break;
                 case BOTTOM: y += height; break;
                 default: break;
                }

  		for (int i = first ; i < last ; i++) {

                   Component m = target.getComponent(i);
		   Dimension md = m.getSize();

                    if (m.isVisible()) {
                                   int px = x + ((width-md.width) / 2);
                                   m.setLocation(x + ((width-md.width) / 2), y );
                                   y += vgap + md.height;
                     }
                    }
      }

    /**
     *
     * @param out
     */
    private void writeObject(java.io.ObjectOutputStream out){
     System.out.println("WRITE...");
    }

    /**
     *
     * @param in
     */
    private Object readObject(java.io.ObjectInputStream in){
     Object tmp=null;

     try{
     System.out.println("READ...");
     tmp = in.readObject();
     }catch(Exception exc){
       exc.printStackTrace();
      }

      return tmp;
    }


}// end VerticalLayoutManager